'*********************************************************************************
'*
'*
'*
'*					Disclaimer
'*
'** This sample script is not supported under any Microsoft standard support program 
'** or service. This sample script is provided AS IS without warranty of any kind. 
'** Microsoft further disclaims all implied warranties including, without limitation,
'** any implied warranties of merchantability or of fitness for a particular purpose.
'** The entire risk arising out of the use or performance of this sample script and
'** documentation remains with you. In no event shall Microsoft, its authors, or 
'** anyone else involved in the creation, production, or delivery of the script be
'** liable for any damages whatsoever (including, without limitation, damages for 
'** loss of business profits, business interruption, loss of business information,
'** or other pecuniary loss) arising out of the use of or inability to use this 
'** sample script or documentation, even if Microsoft has been advised of the 
'** possibility of such damages. 
'**
'**
'**                          Copyright 2003, Microsoft Corporation
'**
'**
'********************************************************************************

'********************************************************************************
'** DESCRIPTION: Sample script showing how to change the RIS Boot file.
'** 
'** REFERENCE: Chapter 4 "Designing RIS Installations" in the Windows Server 2003
'** Deployment Kit: Automating and Customizing Installations
'**
'** SAMPLE Script Name: ChgRISBootFile.txt
'**
'** This sample script shows how to change the default RIS client installation by 
'** changing the names of the startup files on a RIS server. To force RIS clients
'** to perform automated installations, the Startrom.n12 file name is
'** changed to "Startrom.com and Startrom.com is changed to Startrom.bak." To force
'** RIS clients to perform interactive installations, Startrom.com and Startrom.n12
'** are set to their default file name configurations. If RIS clients are not 
'** configured with specific startup files using the sample script PrestagRISClnt.txt,
'** these clients receive the default Startrom.com (interactive) file when 
'** contacting the specified RIS server.
'**
'** Cscript.exe ChgRISBootFile.vbs /Command:[automated | interactive] [/Server:ServerName]
'**
'**   /Command:[automated | interactive]
'**     Specifies whether you want RIS clients to perform automated or interactive
'**     installations. You must specify one of these installation types.
'**
'**  [/Server:ServerName]
'**    Specifies the name of the RIS server on which you want to configure the
'**    default installation type. This parameter is optional. If you do not specify
'**    a RIS server, the local computer is assumed to be the RIS server. If the
'**    local computer is not a RIS server, then the script will fail. Do not use
'**    backslashes (\\) when specifying a server name. 
'**
'********************************************************************************
 OPTION EXPLICIT

 Dim strRetValHst
 Dim intRetVal
 Dim strSpacer
 Dim objArgs
 Dim objNamed
 Dim strCommand
 Dim strServer
 Dim objNetwork
 Dim intFlag

 '** Declare script constants:
     CONST KAUTOMATED    = "automated"
     CONST KINTERACTIVE  = "interactive"
     CONST KUNDETERMINED = "undetermined"
     CONST KERROR        = "error"
     CONST KSUCCESS      = "success"
     CONST KFAILURE      = "failure"
     CONST STRQUOTE      = """"
     CONST KFALSE        =  False
     CONST KTRUE         =  True
     CONST KNOERROR      =  0
     CONST KINVALID_1    =  1
     CONST KINVALID_2    =  2
     CONST KINVALID_3    =  3
     CONST KINVALID_4    =  4
     CONST KINVALID_5    =  5
     CONST KRETVAL_YES   =  6
     CONST KRETVAL_NO    =  7

 ON ERROR RESUME NEXT

 '**
 '** Main execution begins:
 '**

 '** ////////////////////////////////////////////////////
 '** Call the function to detect the default script host:
     strRetValHst = IsHostCScript()

 '** /////////////////////////////////////
 '** Check return value of IsHostCScript():

     Select Case strRetValHst

            Case KERROR
            intRetVal = Msgbox _
            ("The default WSH script host was not detected"           & vbcrlf & _
             "since Error0x" & Hex(Err.Number)                                 & _
             " unexpectedly occurred !"                               & vbcrlf & _
             "Error description: " & Err.Description & vbcrlf         & vbcrlf & _
             "If you continue, the script might fail to execute. . ." & vbcrlf & _
             "Do you want to continue anyway ?", 20, "Script Host Error")

                If intRetVal = KRETVAL_YES Then
            '** Do nothing and proceed with script execution...
                Err.Clear

                ElseIf intRetVal = KRETVAL_NO Then
                Err.Clear
                WScript.Quit()
                End If

            Case KUNDETERMINED
            Msgbox "Could not determine default script host, "                 & _
                   "the script may behave" & vbcrlf & "unexpectedly. . .",       _
                    64, "Script Host Message"

            Case KFAILURE
        '** If default script host is set to WScript, if the script is launched
        '** by double-clicking it, or if user happens to run the script with 
        '** WScript.exe at the command line, then send a message to the user to 
        '** change the default host:

            Msgbox "Script requires CSCRIPT.EXE to execute !"         & vbcrlf & _
                   "Either specify cscript.exe in the command line"   & vbcrlf & _
                   "or change the default script host to cscript.exe" & vbcrlf & _
                   "by running the following command:"       & vbcrlf & vbcrlf & _
                   "cscript.exe //H:cscript //S", 64, "Script Host Message"

            WScript.Quit()

            Case KSUCCESS
        '** Do nothing to proceed...

            Case Else
            Msgbox "Could not determine default script host, "                 & _
                   "the script may behave" & vbcrlf & "unexpectedly. . .",       _
                    64, "Script Host Message"

     End Select


 '** /////////////////////////////////////////////////////////////
 '** Create values for strSpacer variable for formatting messages:

     If Len(Time)     = 10 Then
     strSpacer = "    "
     ElseIf Len(Time) = 11 Then
     strSpacer = "     "
     End If

 '** ////////////////////////////////////////////////////
 '** Create arguments collection and get input arguments:
     Set objArgs  = WScript.Arguments
     Set objNamed = objArgs.Named

     If objArgs.Count = 0 Then
     Call InputError(KINVALID_1)


 '** /////////////////////////////////////
 '** Check if the user is asking for Help:
     ElseIf  objArgs.Item(0) = "?"     OR _
             objArgs.Item(0) = "/?"    OR _
             objArgs.Item(0) = "-h"    OR _
             objArgs.Item(0) = "-H"    OR _
             objArgs.Item(0) = "-help" OR _
             objArgs.Item(0) = "-Help" OR _
             objArgs.Item(0) = "help"  OR _
             objArgs.Item(0) = "Help"  OR _
             objArgs.Item(0) = "/help" OR _
             objArgs.Item(0) = "/Help" Then

      ShowUsage()
      WScript.Quit()         
      End If

 '** ////////////////////
 '** Get RIS server name:
     If objNamed.Exists("Server") Then
     strServer = "\\" & UCase(objNamed("Server"))

     Else
 '** User wants to run script locally on the RIS server:
     Set objNetwork = WScript.CreateObject("WScript.Network")
     strServer = "\\" & UCase(objNetwork.Computername)
     Set objNetwork = Nothing
     End If

     '** Verify correct server name format:
         If strServer = "\\" OR InStr(strServer, "\\\\") <> 0 Then
         Call InputError(KINVALID_2)
         End If

 '** ///////////////////
 '** Get action command:
     If objNamed.Exists("Command") Then
     strCommand = LCase(objNamed("Command"))

     Else
     Call InputError(KINVALID_3)
     End If

 '** //////////////////////////////////////////////////////////////
 '** Check for proper action command and valid value for strServer:
     If strCommand = KINTERACTIVE OR strCommand = KAUTOMATED Then

         If strServer <> "" AND Not IsNull(strServer) Then
         WScript.Echo Time & ": Attempting to configure server " & _
                                strServer & ". . ."
         WScript.Echo ""

         Else
     '** Should not get here, unless strServer is uninitialized or
     '** contains invalid data:
         InputError(KINVALID_4)
         End If

     Else
     Call InputError(KINVALID_5)
     End If

 '** Call procedure to change file names:
     ChangeFileNames()

 '** Release references to arguments objects:
     Set objArgs  = Nothing
     Set objNamed = Nothing

 '**
 '** End main execution
 '**

'********************************************************************************
 Function IsHostCScript()
 
 '** Detects if WSH default host is cscript.exe. .

 Dim strHostPath
 Dim strHostName

 ON ERROR RESUME NEXT

 '** Initialize string variables:
     strHostPath = ""
     strHostName = ""

 '** Get default script host name:
     strHostPath = LCase(WScript.FullName)

 '** Check if any errors occurred (may happen if WSH is not installed):
     If Err.Number <> KNOERROR Then
     IsHostCScript = KERROR
     Exit Function

 '** In case strHostPath variable was not set by WScript.FullName:
     ElseIf strHostPath = "" OR IsEmpty(strHostPath) Then
     IsHostCScript = KUNDETERMINED
     Exit Function

     ElseIf InStr(strHostPath, "cscript") = 0 Then
     IsHostCScript = KFAILURE
     Exit Function

     ElseIf InStr(strHostPath, "cscript") <> 0 Then
     strHostName = Trim(Right(strHostPath, 11))

         If strHostName = "cscript.exe" Then
         IsHostCScript = KSUCCESS

         Else
         IsHostCScript = KFAILURE
         End If

     End If

 End Function

'********************************************************************************
 Sub InputError(intFlag)

 '** Provides abort messages to the user when the input argument configuration is
 '** missing or invalid . . .

     If intFlag     = 1 Then
     WScript.Echo "Aborted, input arguments required, see script Help. ."

     ElseIf intFlag = 2 Then
     WScript.Echo "Invalid server name, please specify correctly, see script Help. ."

     ElseIf intFlag = 3 Then
     WScript.Echo "Input error, command argument required, see script Help. ."

     ElseIf intFlag = 4 Then
     WScript.Echo "Could not interpret the server name properly. ."
     WScript.Quit()

     ElseIf intFlag = 5 Then
     WScript.Echo "Aborted, invalid action command, see script Help. ."

     End If

     WScript.Echo ""
     ShowUsage()
     WScript.Quit()

 End Sub

'********************************************************************************
 Sub ChangeFileNames()

 '** Changes startup boot files on the RIS server according to the specified
 '** action command (either "automated" or "interactive") . . .

 Dim Fso
 Dim blnShareExists
 Dim blnProceed
 Dim objService
 Dim objShare
 Dim Share
 Dim strDefsr
 Dim strChgsr
 Dim strBaksr
 Dim objDefsr
 Dim objChgsr
 Dim objBaksr

 ON ERROR RESUME NEXT

 '** Create file system object:
     Set Fso = CreateObject("Scripting.FileSystemObject")

 '** Initialize boolean variables to false:
     blnShareExists = KFALSE
     blnProceed     = KFALSE

 '** Determine if a remote installation share exists on the server; if so, concatenate 
 '** the server name to the startup boot file paths in the "REMINST" share on the RIS 
 '** server. . .

     If strServer <> "" AND Not IsNull(strServer) Then
     Set objService = GetObject("Winmgmts:" & strServer & "\root\cimv2")

         If Err.Number <> KNOERROR Then
         WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred attempting to " & _
                             "connect with Winmgmts on " & strServer & ". . ."

             If Err.Description > "" Then
             WScript.Echo vbTab & strSpacer & Err.Description & " !"
             End If

         Err.Clear
         WScript.Quit()

         Else
         Set objShare = objService.ExecQuery("Select * from Win32_Share")

             For Each Share in objShare

                 If Share.Name = "REMINST" Then
                 blnShareExists = KTRUE
                 Exit For
                 End If

             Next
     
             If blnShareExists = KTRUE Then
             strDefsr = strServer & "\REMINST\OSchooser\i386\startrom.com"
             strChgsr = strServer & "\REMINST\OSchooser\i386\startrom.n12"
             strBaksr = strServer & "\REMINST\OSchooser\i386\startrom.bak"

             ElseIf blnShareExists = KFALSE Then
             WScript.Echo Time & ": A remote installation share does not exist on " & _
                          strServer & ". . ."                                       & _
                          vbcrlf & vbTab & strSpacer                                & _
                          "If this is your server, you need to install RIS on "     & _
                          "this computer !"

             WScript.Quit()
             End If

         End If

     End If

 '** /////////////////////////////////////////////////////////////////////
 '** Verify input command against the existing startup file configuration
 '** on the RIS server:

     If strCommand = KINTERACTIVE Then

         If (Fso.FileExists(strDefsr)) AND (Fso.FileExists(strChgsr)) Then
         WScript.Echo vbTab & strSpacer & "Startup configuration already set for " & _
                      STRQUOTE & "interactive" & STRQUOTE & " mode on "            & _
                      strServer & " !"

         WScript.Quit()

     '** If strDefsr and strBaksr exist, automated mode was previously set, so 
     '** we set blnProceed = True to proceed to change back to interactive mode:

         ElseIf (Fso.FileExists(strDefsr)) AND (Fso.FileExists(strBaksr)) Then
         blnProceed = KTRUE

         Else
     '** One or both files are missing:
         Call DisplayMsg(strDefsr, strChgsr)

         End If

     ElseIf strCommand = KAUTOMATED Then

         If (Fso.FileExists(strDefsr)) AND (Fso.FileExists(strBaksr)) Then
         WScript.Echo vbTab & strSpacer & "Startup configuration already set for " & _
                      STRQUOTE & "automated" & STRQUOTE & " mode on "              & _
                      strServer & " !"

         WScript.Quit()

     '** If strDefsr and strChgsr exist, the interactive mode is set, so 
     '** we set blnProceed = True to proceed to change to the automated mode:

         ElseIf (Fso.FileExists(strDefsr)) AND (Fso.FileExists(strChgsr)) Then
         blnProceed = KTRUE

         Else
     '** One or both files are missing:
         Call DisplayMsg(strDefsr, strChgsr)

         End If

     End If

 '** If proceed flag is set to true, change the file names:
     If blnProceed = KTRUE Then

         If strCommand = KAUTOMATED Then

             Set objDefsr  = Fso.GetFile(strDefsr)
             objDefsr.Name = "startrom.bak"

             Set objChgsr  = Fso.GetFile(strChgsr)
             objChgsr.Name = "startrom.com"

         ElseIf strCommand = KINTERACTIVE Then

             Set objDefsr  = Fso.GetFile(strDefsr)
             objDefsr.Name = "startrom.n12"

             Set objBaksr  = Fso.GetFile(strBaksr)
             objBaksr.Name = "startrom.com"

         End If

     End If

 '** ////////////////////////////////
 '** Check for any unexpected errors:

     If Err.Number <> KNOERROR Then
     WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred unexpectedly. ." & _
                            vbcrlf & vbTab & strSpace                                & _
                            "Please change the file names manually!"

         If Err.Description > "" Then
         WScript.Echo vbTab & strSpacer & Err.Description
         End If

     Err.Clear

     Else
     WScript.Echo Time & ": Success configuring startup files on " & strServer & " for " & _
                            STRQUOTE & strCommand & STRQUOTE & " installations !"   
     End If

'** Release object references:
    
    Set objBaksr   = Nothing
    Set objDefsr   = Nothing
    Set objChgsr   = Nothing
    Set objService = Nothing
    Set objShare   = Nothing
    Set Fso        = Nothing

 End Sub
 
'********************************************************************************
 Sub DisplayMsg(strDefsr, strChgsr)

 WScript.Echo Time & ": One or both of the following files do not exist:"      & _
                        vbcrlf & vbcrlf                                        & _
                        vbtab & strSpacer & Right(strDefsr, 12)                & _
                        vbcrlf                                                 & _
                        vbtab & strSpacer & Right(strChgsr, 12)                & _
                        vbcrlf & vbcrlf                                        & _
                        "Please create at least one Risetup image "            & _
                        "to generate these files. . ."
 WScript.Quit()

 End Sub

'********************************************************************************
 Sub ShowUsage()

 WScript.Echo "******************************* ChgRISBootFile **************************"
 WScript.Echo ""
 WScript.Echo "SCRIPT HELP. . ."
 WScript.Echo ""
 WScript.Echo "PURPOSE:"
 WScript.Echo "Changes the file names of startrom.com and startrom.n12 on a specified"
 WScript.Echo "RIS server to enable automated or interactive installations. Use the"
 WScript.Echo """automated"" command to provide automated installations for all clients"
 WScript.Echo "contacting the RIS server."
 WScript.Echo ""
 WScript.Echo "APPLICATION:"
 WScript.Echo "Run script on RIS server or from RIS domain computer."
 WScript.Echo ""
 WScript.Echo "OPTIONS:"
 WScript.Echo "/Command:Value   Value specifies the action command you want to run:"
 WScript.Echo "                 ""automated"" ---- changes startrom.n12 to startrom.com."
 WScript.Echo "                 ""interactive"" -- returns to the default startrom.com."
 WScript.Echo "/Server:Value    Value specifies RIS server where the boot files reside."
 WScript.Echo "                 If server is unspecified, local computer is assumed."
 WScript.Echo ""
 WScript.Echo "USAGE EXAMPLE:"
 WScript.Echo "cscript.exe ChgRISBootFile.vbs /Command:automated [/Server:servername]"
 WScript.Echo ""
 WScript.Echo "NOTES:"
 WScript.Echo "1. Do not use double backslashes (\\) when specifying servername."
 WScript.Echo "2. Admin privileges required in the RIS domain."
 WScript.Echo "3. Script requires WSH version 5.6."
 WScript.Echo ""
 WScript.Echo "**************************************************************************"

 End Sub


