'*********************************************************************************
'*
'*
'*
'*					Disclaimer
'*
'** This sample script is not supported under any Microsoft standard support program 
'** or service. This sample script is provided AS IS without warranty of any kind. 
'** Microsoft further disclaims all implied warranties including, without limitation,
'** any implied warranties of merchantability or of fitness for a particular purpose.
'** The entire risk arising out of the use or performance of this sample script and
'** documentation remains with you. In no event shall Microsoft, its authors, or 
'** anyone else involved in the creation, production, or delivery of the script be
'** liable for any damages whatsoever (including, without limitation, damages for 
'** loss of business profits, business interruption, loss of business information,
'** or other pecuniary loss) arising out of the use of or inability to use this 
'** sample script or documentation, even if Microsoft has been advised of the 
'** possibility of such damages. 
'**
'**
'**                          Copyright 2003, Microsoft Corporation
'**
'**
'********************************************************************************

'********************************************************************************
'** DESCRIPTION: Sample script showing how to retrieve BIOS information for prestaging 
'** RIS Clients.
'**
'** REFERENCE: Chapter 4 "Designing RIS Installations" in the Windows Server 2003
'** Deployment Kit: Automating and Customizing Installations
'**
'** SAMPLE Script Name: GetRISBIOSInfo.txt
'**
'** This sample script shows how you can retrieve UUIDs/GUIDs and BIOS data from either
'** the Computer container in the Active Directory (AD) directory service or from
'** individual computers. The UUIDs/GUIDs can be used to prestage computers for RIS 
'** installations. The BIOS data can be used to determine whether a computer is 
'** PXE-capable. 
'**
'** To retrieve UUIDs/GUIDs from the AD Computer container, each computer must have 
'** location and description parameters configured in their Active Directory
'** computer accounts.
'**
'** By default, the script echoes output to the command line. If you are retrieving
'** UUIDs/GUIDs from the AD Computer container, you can have the script write the information
'** to an Excel spreadsheet. An option is also available to write events to the Event
'** log if there is an error retrieving a UUID. 
'**
'** SYNTAX:
'**
'** Cscript.exe GetRISBIOSInfo.vbs /Command:[getuuid|getalluuids|getbios|getallbios]
'**   [/Server:ServerName] [/ExcelPath:PathToSpreadsheet] [/LogEvent:ServerName]
'**
'**   /Command:[getuuid|getalluuids|getbios|getallbios]
'**     getuuid		Returns UUID for a single computer
'**     getalluuids	Returns UUIDs for all computers in AD Computers container
'**     getbios		Returns BIOS data for a single computer
'**     getallbios	Returns BIOS data for all computers in AD Computers container
'**
'**   /Server:ServerName
'**     Specifies the name of the computer for which you want to get UUID/GUID or BIOS
'**     information. Do not use \\ in computer name. This option is only used with
'**     the getuuid and getbios commands. If a computer name is not specified, the
'**     local computer is assumed.
'**
'**   /ExcelPath:PathToSpreadsheet
'**     Specifies the location and name of the spreadsheet that you want to write
'**     data to. This option can only be used with the getalluuids command.
'**
'**   /LogEvent:ServerName
'**     Specifies the name of a computer where the script generates event log entries
'**     if errors occur or if UUID information is unavailable. This options can
'**     only be used with the getalluuids command.
'**
'********************************************************************************
 OPTION EXPLICIT

 Dim strRetValHst
 Dim intRetVal
 Dim objWshNetwork
 Dim strCurrentUser
 Dim strSpace
 Dim objArgs
 Dim objNamed
 Dim intFlag
 Dim strCommand
 Dim strExcelPath
 Dim strExcelFlg
 Dim strServer
 Dim strCmdFlag
 Dim objDict
 Dim intKey
 Dim strLogTarget

 '** Declare script constants:
     CONST KNOERROR       = 0
     CONST KINVALID_1     = 1
     CONST KINVALID_2     = 2
     CONST KINVALID_3     = 3
     CONST KINVALID_4     = 4
     CONST KINVALID_5     = 5
     CONST KRETVAL_YES    = 6
     CONST KRETVAL_NO     = 7
     CONST KUNDETERMINED  = "undetermined"
     CONST KERROR         = "error"
     CONST KSUCCESS       = "success"
     CONST KFAILURE       = "failure"
     CONST KENABLE        = "enable"
     CONST KDISABLE       = "disable"
     CONST KUUID          = "uuidsearch"
     CONST KBIOS          = "biossearch"
     CONST KQUOTE         = """"

 ON ERROR RESUME NEXT

 '**
 '** Main execution begins
 '**

 '** ////////////////////////////////////////////////////
 '** Call the function to detect the default script host:
     strRetValHst = IsHostCScript()

 '** /////////////////////////////////////
 '** Check return value of IsHostCScript():

     Select Case strRetValHst

            Case KERROR
            intRetVal = Msgbox _
            ("The default WSH script host was not detected"           & vbcrlf & _
             "since Error0x" & Hex(Err.Number)                                 & _
             " unexpectedly occurred !"                               & vbcrlf & _
             "Error description: " & Err.Description & vbcrlf         & vbcrlf & _
             "If you continue, the script might fail to execute. . "  & vbcrlf & _
             "Do you want to continue anyway ?", 20, "Script Host Error")

                If intRetVal = KRETVAL_YES Then
            '** Do nothing and proceed with script execution...
                Err.Clear

                ElseIf intRetVal = KRETVAL_NO Then
                Err.Clear
                WScript.Quit()
                End If

            Case KUNDETERMINED
            Msgbox "Could not determine default script host, "                 & _
                   "the script may behave" & vbcrlf & "unexpectedly. . .",       _
                    64, "Script Host Message"

            Case KFAILURE

        '** If default script host is set to WScript, if the script is launched
        '** by double-clicking it, or if user happens to run the script with 
        '** WScript.exe at the command line, then send a message to the user to 
        '** change the default host:

            Msgbox "Script requires CSCRIPT.EXE to execute !"         & vbcrlf & _
                   "Either specify cscript.exe in the command line"   & vbcrlf & _
                   "or change the default script host to cscript.exe" & vbcrlf & _
                   "by running the following command:"       & vbcrlf & vbcrlf & _
                   "cscript.exe //H:cscript //S", 64, "Script Host Message"

            WScript.Quit()

            Case KSUCCESS
        '** Do nothing to proceed...

            Case Else
            Msgbox "Could not determine default script host, "                 & _
                   "the script may behave" & vbcrlf & "unexpectedly. . .",       _
                    64, "Script Host Message"

     End Select

 '** //////////////////////////////////////
 '** Create spacer for formatting messages:

     If Len(Time)     = 10 Then
     strSpace = "    "
     ElseIf Len(Time) = 11 Then
     strSpace = "     "
     End If

 '** /////////////////////////////////////////////////////
 '** Create global WshNetwork object and get current user:
     Set objWshNetwork = WScript.CreateObject("WScript.Network")
     strCurrentUser    = objWshNetwork.UserName

 '** /////////////////////////////////////////////
 '** Create Wsh Named arguments collection object:

     Set objArgs  = WScript.Arguments
     Set objNamed = objArgs.Named

 '** /////////////////////////////////////
 '** Check if the user is asking for Help:
     If objArgs.Count <> 0 Then

         If objArgs.Item(0) = "?"     OR _
            objArgs.Item(0) = "/?"    OR _
            objArgs.Item(0) = "-h"    OR _
            objArgs.Item(0) = "-H"    OR _
            objArgs.Item(0) = "-help" OR _
            objArgs.Item(0) = "-Help" OR _
            objArgs.Item(0) = "help"  OR _
            objArgs.Item(0) = "Help"  OR _
            objArgs.Item(0) = "/help" OR _
            objArgs.Item(0) = "/Help" Then

         ShowUsage()
         ArgObjCleanup()
         WScript.Quit()
         End If

     ElseIf objArgs.Count = 0 Then
     ArgObjCleanup()
 '** Call procedure to display message alerting user to invalid input argument
 '** configuration:
     Call InputError(KINVALID_1)
     End If

 '** /////////////////////////////////////////////
 '** Detect the main input argument configuration:

 '** Initialize string variable to be set by input action command:
     strCommand = ""

 '** Get action command for calling the appropriate procedure:
     If objNamed.Exists("Command") Then
     strCommand = LCase(objNamed("Command"))

     Else
     ArgObjCleanup()
     Call InputError(KINVALID_3)
     End If

 '** Check action commands entered by the user and call the appropriate 
 '** procedure:

 '** Initialize string-type command flag:
     strCmdFlag = ""

     If strCommand = "getalluuids" Then
 '** User wants to return UUIDs for all computers in Computers container of 
 '** Active Directory. . 

     '** Check if an Excel spreadsheet to print UUIDs/GUIDs is being requested:
         If objNamed.Exists("ExcelPath") Then
         strExcelPath = objNamed("ExcelPath")

             If InStr(strExcelPath, ".xls") <> 0 Then
         '** Set strExcelFlg value to "enable":
             strExcelFlg = KENABLE

             Else
             ArgObjCleanup()
             Call InputError(KINVALID_2)
             End If

         Else
     '** Explicitly set flag value to "disable":
         strExcelFlg = KDISABLE
         End If

     '** Check if a server name was specified to gather Event Log entries:
         If objNamed.Exists("LogEvent") Then
         strLogTarget = "\\" & objNamed("LogEvent")

             If strLogTarget = "\\" OR InStr(strLogTarget, "\\\\") <> 0 Then
             ArgObjCleanup()
             Call InputError(KINVALID_4)
             End If

         Else
     '** Default to the local computer:
         strLogTarget = ""
         End If

 '** Set a global command flag and call procedure to get computer accounts:
     strCmdFlag = KUUID
     Call GetCompAccts()

     ElseIf strCommand = "getallbios" Then
 '** User wants to return BIOS parameters for all computers in Computers container
 '** of Active Directory, so we set a global flag and call procedure to get computer 
 '** accounts:

     strCmdFlag = KBIOS
     Call GetCompAccts()

     ElseIf strCommand = "getuuid" Then
 '** User wants to return the UUID for a specified server...

 '** Call procedure to get server name:
     Call GetServer(strServer)

         If strServer > "" AND strServer <> "\\" AND InStr(strServer, "\\\\") = 0 Then
     '** Call procedure to get the UUID on the specified server:
         Call GetSystemUUID(strServer, objDict, intKey)

         Else
         ArgObjCleanup()
         Call InputError(KINVALID_4)
         End If

     ElseIf strCommand = "getbios" Then
 '** User wants to return BIOS parameters for a specified server...

     Call GetServer(strServer)

         If strServer > "" AND strServer <> "\\" AND InStr(strServer, "\\\\") = 0 Then
     '** Call procedure to get BIOS parameters on the specified server:
         Call GetBiosProps(strServer)

         Else
         ArgObjCleanup()
         Call InputError(KINVALID_4)
         End If

     Else
     ArgObjCleanup()
     Call InputError(KINVALID_5)
     End If

     ArgObjCleanup()

'**
'** Main execution ends
'**

'********************************************************************************
 Function IsHostCScript()
 
 '** Detects if WSH default host is cscript.exe. .

 Dim strHostPath
 Dim strHostName

 ON ERROR RESUME NEXT

 '** Initialize string variables:
     strHostPath = ""
     strHostName = ""

 '** Get default script host name:
     strHostPath = LCase(WScript.FullName)

 '** Check if any errors occurred (may happen if WSH is not installed):
     If Err.Number <> KNOERROR Then
     IsHostCScript = KERROR
     Exit Function

 '** In case strHostPath variable was not set by WScript.FullName:
     ElseIf strHostPath = "" OR IsEmpty(strHostPath) Then
     IsHostCScript = KUNDETERMINED
     Exit Function

     ElseIf InStr(strHostPath, "cscript") = 0 Then
     IsHostCScript = KFAILURE
     Exit Function

     ElseIf InStr(strHostPath, "cscript") <> 0 Then
     strHostName = Trim(Right(strHostPath, 11))

         If strHostName = "cscript.exe" Then
         IsHostCScript = KSUCCESS

         Else
         IsHostCScript = KFAILURE
         End If

     End If

 End Function

'********************************************************************************
 Sub InputError(intFlag)

 '** Called when the Named argument collection is being processed and the user
 '** failed to provide the proper argument configuration or input values.
 '** Echoes messages to the command line and displays script usage instructions, 
 '** if the user fails to specify an action command, provides an invalid 
 '** server name, or if the user specifies an invalid action command. .

     If intFlag     = 1 Then
     WScript.Echo "Aborted, input arguments required, see script Help. ."

     ElseIf intFlag = 2 Then
     WScript.Echo "Aborted, please specify the .xls extension for your Excel file !"
     WScript.Quit()

     ElseIf intFlag = 3 Then
     WScript.Echo "Aborted, command argument required, see script Help. ."

     ElseIf intFlag = 4 Then
     WScript.Echo "Aborted, invalid server name, see script Help. ."

     ElseIf intFlag = 5 Then
     WScript.Echo "Aborted, invalid command argument, see script Help. ."
 
     End If

     WScript.Echo ""
     ShowUsage()

 '** If this procedure was called, it is because the user provided an invalid
 '** input argument configuration, so we quit the script:

     WScript.Quit()

 End Sub

'********************************************************************************
 Sub GetServer(ByRef strServer)

 '** Gets the name of remote server specified at the command line or if server
 '** name is not specified, gets the name of the local machine. .

     If objNamed.Exists("Server") Then
     strServer = "\\" & objNamed.Item("Server")

     Else
 '** Use global wshNetwork object to get name of local computer:
     strServer = "\\" & LCase(objWshNetwork.ComputerName)
     Set objWshNetwork = Nothing
     End If

 End Sub

'********************************************************************************
 Sub ArgObjCleanup()

 '** Releases references to arguments objects. . 

     Set objArgs  = Nothing
     Set objNamed = Nothing

 End Sub

'********************************************************************************
 Sub GetCompAccts()

 '** Queries Active Directory for computer account objects in the default
 '** Computers container, adds them to a dictionary object, and enumerates the
 '** values of an associative array (dictionary) to call the procedures that
 '** return the requested data from the specified machines (computer accounts). .

 Dim Fso
 Dim rootDSE
 Dim compContainer
 Dim dnArray
 Dim intPos
 Dim strDomain
 Dim objADsComp
 Dim Item
 Dim objDict
 Dim i
 Dim Computer
 Dim objColDict
 Dim objKeys
 Dim intKey

 CONST ADS_GUID_COMPUTRS_CONTAINER = "aa312825768811d1aded00c04fd8d5cd"

 ON ERROR RESUME NEXT

 Set Fso = CreateObject("Scripting.FileSystemObject")

 '** ///////////////////////////////////////////////////////////////////
 '** Create instance of Active Directory Computers container: locate the 
 '** domain controller for the domain, read the distinguished name, and 
 '** bind to the default "Computers" container:

     Set rootDSE = GetObject("LDAP://RootDSE")
     Set compContainer = GetObject("LDAP://<WKGUID=" & ADS_GUID_COMPUTRS_CONTAINER & _
                                   "," & rootDSE.Get("defaultNamingContext") & ">")
     Set objADsComp = GetObject("LDAP://" & compContainer.Get("distinguishedName"))
 
 '** Check for errors getting instance of Active Directory "Computers" object:
     If Err.Number <> KNOERROR Then
     WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred geting instance " & _
                  "of Active Directory default Computers container. . ."

         If Err.Description <> "" Then
         WScript.Echo vbTab & strSpace & Err.Description
         End If

     Err.Clear
     WScript.Quit()

     Else

 '** Get domain name and echo message to command line:
     dnArray = Split(compContainer.Get("DistinguishedName"), ",")
     intPos = InStr(dnArray(1), "=")

         If intPos <> 0 Then
         strDomain = KQUOTE & UCase(Mid(dnArray(1), intPos + 1, Len(dnArray(1)) - intPos + 1)) & KQUOTE
         
         Else
     '** Shouldn't get here:
         strDomain = "primary"
         End If

     WScript.Echo Time & ": Searching Active Directory default Computers container " & _
                  "in the " & strDomain & " domain. . ."
     WScript.Echo ""
     End If

 '** ////////////////////////////////////////////////////////////////
 '** Create dictionary object to store computer account names, UUIDs,
 '** and computer location and description attributes:

     Set objDict = CreateObject("Scripting.Dictionary")

 '** Add computer names, location, and description to the dictionary object; 
 '** UUIDs will be added later in GetSystemUUID (to the i + 1 placeholder
 '** key that is initialized here to a zero-length string):

     i = 0

     Err.Clear
     For Each Computer In objADsComp

     objDict.Add i + 0, Computer.Get("Name")
     objDict.Add i + 1, ""
     objDict.Add i + 2, Computer.Get("Location")

     '** If the computer account does not have a location parameter configured,
     '** "Location" will not be in the property cache, so the last line
     '** generates an error and the dictionary key/value is not set. So we 
     '** configure "unspecified" in this dictionary key for the computer:
         If Err.Number <> KNOERROR Then
         objDict.Add i + 2, "Unspecified"
         Err.Clear
         End If

     objDict.Add i + 3, Computer.Get("Description")

     '** Same as above for Description property:
         If Err.Number <> KNOERROR Then
         objDict.Add i + 3, "Unspecified"
         Err.Clear
         End If

     i = i + 4

     Next

 '** If the count of dictionary items is 0, no computer accounts could be 
 '** found, so we quit:
     If objDict.Count = 0 Then
     WScript.Echo Time & ": Could not find any computer accounts in the" & _
                         " default Computers container of Active Directory. . ."
     WScript.Quit()
     End If

 '** ///////////////////////////////////////////////////////////////
 '** Create a variable for enumerating the dictionary collection and 
 '** calling procedures to obtain requested data:
     objColDict = objDict.Items

 '** Initialize the variable intKey, which is passed to the procedure 
 '** GetSystemUUID and used to provide indexing for adding UUID values  
 '** to the dictionary object. In the For Loop, intKey is set to the 
 '** dictionary key values that are placeholders for the UUID
 '** data (1, 5, 9, etc) for each strServer in the dictionary:
     intKey = 0

     For i = 0 to objDict.Count - 1 Step 4
     strServer = "\\" & objColDict(i)
     intKey =  i + 1

         If strServer > "" AND strServer <> "\\" Then

         '** Call appropriate procedures based on value of the global command flag:
             If strCmdFlag = KUUID Then
             Call GetSystemUUID(strServer, objDict, intKey)

             ElseIf strCmdFlag = KBIOS Then
             Call GetBiosProps(strServer)

             Else
         '** Should never get here:
             WScript.Echo Time & ": The action command was incorrectly interpreted !"
             WScript.Quit()
             End If

         End If

     Next

 '** ///////////////////////////////////////////////////////////////////////
 '** If the user is asking to render the output on an Excel spreadsheet, the
 '** variable strExcelFlg will be set to "enable", so we call the procedure 
 '** MakeExcel:

     If strCmdFlag = KUUID AND strExcelFlg = KENABLE Then
     Call MakeExcel(objDict, strDomain)
     End If

 '** ////////////////////////////
 '** Check for unexpected errors:

     If Err.Number = KNOERROR Then

         If strExcelFlg = KENABLE Then

             If (Fso.FileExists(strExcelPath)) Then

             Msgbox "Created Excel spreadsheet specifying "       & _
                    "UUIDs for computer systems in the"  & vbcrlf & _
                     strDomain & " Active Directory domain. "     & _
                    "The path to the spreadsheet is. . ."         & _
                     vbcrlf & vbcrlf & UCase(strExcelPath), 64,     _
                    "Excel Output"
             End If

         End If

     WScript.Echo Time & ": Finished retrieving multiple data sets !"

     Else
     WScript.Echo Time & ": An unexpected error occurred while servicing your request !"

         If Err.Description <> "" Then
         WScript.Echo vbTab & strSpace & Err.Description
         End If

     Err.Clear
     End If

 '** Release object references:
     Set objDict    = Nothing
     Set objADsComp = Nothing
     Set Fso        = Nothing

 End Sub

'********************************************************************************
 Sub GetSystemUUID(strServer, objDict, intKey)

 Dim objWshShell
 Dim objService
 Dim objSysComps
 Dim blnWFMCapable
 Dim SysItem
 Dim strUUID
 Dim strOut
 Dim intUUIDCount
 Dim NetworkAdapterSet
 Dim NicItem
 Dim strAdapterDescription
 Dim strMACAdd

 ON ERROR RESUME NEXT

 Set objWshShell = WScript.CreateObject("WScript.Shell")
 Set objService  = GetObject("winmgmts:" & strServer & "\root\cimv2")

 '** Check for successful object retrieval from CIM repository on specified server, 
 '** through the Winmgmts connection:
     If Err.Number <> KNOERROR Then

 '** Log an Event Log entry on the computer specifed by strLogTarget:
     objWshShell.LogEvent 1, "UUID RETRIEVAL ERROR: Couldn't access Winmgmts on "     & _
                              UCASE(strServer) & " since Error0x" & Hex(Err.Number)   & _
                             " occurred. A UUID could not be returned for this"       & _
                             " computer!", strLogTarget

         If strExcelFlg <> KENABLE Then
         WScript.Echo Time & ": Couldn't access Winmgmts on " & KQUOTE                & _
                                UCase(strServer) & KQUOTE & " ; Error0x"              & _
                                Hex(Err.Number) & " occurred. . ."
                                  
             If Err.Description <> "" Then
             Wscript.Echo vbtab & strSpace & "Description: " & Err.Description & "."
             End If


             WScript.Echo vbtab & strSpace & "Cannot return UUID on "                 & _
                          UCASE(strServer) & "; please check the name of the server"  & _
                          vbcrlf & vbTab & strSpace & "or domain you specified, and Wbem !"         
             WScript.Echo ""

         End If

     '** Add a dictionary entry indicating that the UUID/GUID is undetermined for
     '** the computer being processed for this call, then exit:
         If strExcelFlg = KENABLE Then
         objDict.Remove intKey
         objDict.Add intKey, "Undetermined"
         End If

     Err.Clear
     Set objWshShell = Nothing
     Set objService  = Nothing
     Exit Sub

     ElseIf Err.Number = KNOERROR Then

         If strExcelFlg <> KENABLE Then
         WScript.Echo Time & ": Connected with Winmgmts on " & KQUOTE    & _
                                UCase(strServer) & KQUOTE                & _
                              " and accessing computer system. . ."
         End If

     End If

 '** Get instance of Win32_ComputerSystemProduct class (collection object):
     Set objSysComps = objService.ExecQuery("Select * from Win32_ComputerSystemProduct")

 '** Initialize a boolean-typed variable used to detect the presence of
 '** a valid UUID in the collection object objSysComps:
     blnWFMCapable = False

 '** Initialize counter variable that detects number of UUIDs found:
     intUUIDCount = 0

     For Each SysItem in objSysComps
     strUUID = SysItem.UUID

      '** If we find one or more valid UUIDs/GUIDs in the collection, build up a string
      '** containing the UUIDs/GUIDs and set a boolean-typed flag to true. However,
      '** there will probably be only one item in the collection for the
      '** Win32_ComputerSystemProduct class:
          
          If Not IsEmpty(strUUID) OR Not IsNull(strUUID) Then

              If strUUID <> "00000000-0000-0000-0000-000000000000" AND _
                 strUUID <> "FFFFFFFF-FFFF-FFFF-FFFF-FFFFFFFFFFFF" Then
              strOut = strOut & strUUID & vbLf
              intUUIDCount = intUUIDCount + 1
              blnWFMCapable = True
              End If

          End If

      Err.Clear
      Next

      Set objSysComps = Nothing
      Set objService = Nothing

 '** ///////////////////////////
 '** Check results of iteration:

     If Not blnWFMCapable AND intUUIDCount = 0 Then

 '** strUUID returns a value of "00000000-0000-0000-0000-000000000000" or 
 '** "FFFFFFFF-FFFF-FFFF-FFFF-FFFFFFFFFFFF" if a valid UUID cannot be found, 
 '** so we create one based on the MAC address of the network adapter. . .

     Set NetworkAdapterSet = GetObject("winmgmts:" & strServer & _
                             "\root\cimv2").InstancesOf("Win32_NetworkAdapter")
 
         For Each NicItem In NetworkAdapterSet

         If NicItem.AdapterType = "Ethernet 802.3" Then
         strAdapterDescription = NicItem.Description

             If strAdapterDescription <> "Packet Scheduler Miniport" Then
             strMACAdd = NicItem.MACAddress
             strUUID   = "00000000-0000-0000-0000-" & Replace(strMACAdd, ":", "")

             objWshShell.LogEvent 0, _
             "UUID RETRIEVAL: A UUID could not be found on " & strServer & "."         & _
              vbcrlf & "This computer system does not appear to support Intel's "      & _
             """Wired For Management"" specification. Based on the network "           & _
             "adapter with the description " & KQUOTE & strAdapterDescription & KQUOTE & _
             ", a MAC-based UUID was derived that you can use to prestage this "       & _
             "client computer in Active Directory:" & vbcrlf & vbcrlf                  & _
              strUUID, strLogTarget

                 If strExcelFlg <> KENABLE Then
                 WScript.Echo ""
                 WScript.Echo vbTab & strSpace & "A system UUID could not be found. This system does not appear to support"      & vbcrlf & _
                              vbTab & strSpace & "Intel's ""Wired For Management"" specification. Based on the network"          & vbcrlf & _
                              vbTab & strSpace & "adapter with the description " & KQUOTE & strAdapterDescription & KQUOTE & "," & vbcrlf & _ 
                              vbTab & strSpace & "a MAC-based UUID was derived for prestaging this client in Active Directory:"  & vbcrlf & _
                              vbcrlf & vbTab & strSpace & strUUID
                 WScript.Echo ""
                 End If

             Exit For
             End If

         End If

         Next
         Set NetworkAdapterSet = Nothing

     ElseIf blnWFMCapable Then

         If strExcelFlg <> KENABLE Then

             If intUUIDCount = 1 Then
             WScript.Echo vbTab & strSpace & "Success retrieving system UUID:  " _
                          & strOut

         '** Should not get here:
             ElseIf intUUIDCount > 1 Then
             WScript.Echo vbTab & strSpace & "Multiple system UUIDs were found:" _
                          & vbcrlf & vbTab & strSpace & strOut
             End If

         End If

     End If

 '** Add the native UUID/GUID or derived UUID/GUID to the dictionary in the appropriate key 
 '** position:
     If strExcelFlg = KENABLE AND strCmdFlag = KUUID Then

         If intUUIDCount > 1 Then
     '** Should not get here:
         objDict.Remove intKey
         objDict.Add intKey, "Multiple UUIDs found:" & vbLf & strOut

         ElseIf intUUIDCount <= 1 Then
     '** Case where a single UUID was found (intUUIDCount = 1); case where a UUID was 
     '** derived from the MAC address (intUUIDCount = 0):
         objDict.Remove intKey
         objDict.Add intKey, strUUID
         End If

     End If

     Set objWshShell = Nothing
     Set objService  = Nothing

 End Sub

'********************************************************************************
 Sub GetBiosProps(strServer)
 
 Dim objWinMgmts
 Dim objBios
 Dim Property
 Dim intItem
 Dim strBiosChar
 Dim strBiosVersion
 Dim intOut

 ON ERROR RESUME NEXT

 Set objWinMgmts = GetObject("Winmgmts:" & strServer & "\root\cimv2")
 
 '** Check for errors while connecting to the CIM repository:
     If Err.Number <> KNOERROR Then
     WScript.Echo Time & ": Couldn't access Winmgmts on " & KQUOTE & UCase(strServer) & KQUOTE & _
                         " ; Error0x" & Hex(Err.Number) & " occurred. . ."
                                  
        If Err.Description <> "" Then
        Wscript.Echo vbtab & strSpace & "Description: " & Err.Description & "."
        End If

     WScript.Echo vbtab & strSpace & "Cannot return BIOS properties on " & _
                  UCASE(strServer) & "; please check the name " & vbcrlf & _
                  vbTab & strSpace & "of the server or domain you "      & _
                 "specified, and Wbem !"
     WScript.Echo ""

     Err.Clear
     Set objWinMgmts = Nothing
     Exit Sub

     ElseIf Err.Number = KNOERROR Then
     WScript.Echo Time & ": Connected with Winmgmts on " & KQUOTE & _
                            UCase(strServer) & KQUOTE             & _
                          " and accessing computer system. . ."
     WScript.Echo ""

 '** Get instance of Win32_BIOS class:
     Set objBios = objWinMgmts.ExecQuery("Select * from Win32_BIOS")

     For Each Property in objBios
     
     '** Filter for the BIOSCharacteristics properties:
         If IsArray(Property.BiosCharacteristics) Then

         For Each intItem in Property.BiosCharacteristics

     '** Test the return values combination:
     '** intOut = intOut & intItem & "  "

             Select Case intItem

                    Case 2
                    strBiosChar = strBiosChar & "unknown" & vbcrlf

                    Case 3
                    strBiosChar = strBiosChar & "bios characteristics not supported" & vbcrlf

                    Case 7
                    strBiosChar = strBiosChar & "pci is supported" & vbcrlf

                    Case 8
                    strBiosChar = strBiosChar & "pc card (pcmcia) is supported" & vbcrlf

                    Case 9
                    strBiosChar = strBiosChar & "plug and play is supported" & vbcrlf

                    Case 11
                    strBiosChar = strBiosChar & "bios is upgradeable (flash)" & vbcrlf

                    Case 16
                    strBiosChar = strBiosChar & "selectable boot is supported" & vbcrlf

                    Case 18
                    strBiosChar = strBiosChar & "boot from pc card (pcmcia) is supported" & vbcrlf

             End Select
 
         Next

         End If


     '** ///////////////////////////////////////////////
     '** Detect BIOS parameters of interest and display:

         strBiosVersion = CStr(Property.SmBiosMajorVersion) & "." & _
                          CStr(Property.SMBiosMinorVersion)
     
         WScript.Echo "Parameters for " & Property.Version & ":"
         WScript.Echo ""
         WScript.Echo "SMBIOS version:                        " & strBiosVersion

     '** Check if BIOS characteristics is suppported or unknown:
         If InStr(LCase(strBiosChar), "bios characteristics not supported") <> 0 OR _
         InStr(LCase(strBiosChar), "unknown") <> 0 Then
         WScript.Echo "Characteristics for this BIOS cannot be returned . . ."
         End If

     '** Check if the BIOS supports PCI network adapters:
         If InStr(LCase(strBiosChar), "pci is supported") <> 0 Then
         WScript.Echo "BIOS supports PCI network adapters:    Yes"
         Else
         WScript.Echo "BIOS supports PCI network adapters:    No"
         End If

     '** Check if the BIOS supports selectable boot:
         If InStr(LCase(strBiosChar), "selectable boot is supported") <> 0 Then
         WScript.Echo "BIOS supports selectable booting:      Yes"
         Else
         WScript.Echo "BIOS supports selectable booting:      No"
         End If

     '** Check if the BIOS supports plug and play:
         If InStr(LCase(strBiosChar), "plug and play is supported") <> 0 Then
         WScript.Echo "BIOS supports plug and play:           Yes"         
         Else
         WScript.Echo "BIOS supports plug and play:           No"
         End If

     '** Check if the BIOS supports PC Card network adapters:
         If InStr(LCase(strBiosChar), "pc card (pcmcia) is supported") <> 0 Then
         WScript.Echo "BIOS supports PC Card network adapter: Yes"
         Else
         WScript.Echo "BIOS supports PC Card network adapter: No"
         End If

     '** Check if the BIOS supports booting from PC Card:
         If InStr(LCase(strBiosChar), "boot from pc card (pcmcia) is supported") <> 0 Then
         WScript.Echo "BIOS supports boot from PC Card:       Yes"
         Else
         WScript.Echo "BIOS supports boot from PC Card:       No"
         End If

     '** Check if the BIOS is upgradeable:
         If InStr(LCase(strBiosChar), "bios is upgradeable (flash)") <> 0 Then
         WScript.Echo "BIOS is upgradeable (flash):           Yes"             
         Else
         WScript.Echo "BIOS is upgradeable (flash):           No"
         End If

         WScript.Echo ""

     '** WScript.Echo "Values: " & intOut 

     Next

     End If

     '** Release object references:
         Set objWinMgmts = Nothing
         Set objBios     = Nothing

 End Sub

'********************************************************************************
 Sub MakeExcel(objDict, strDomain)

 '** Prepares an Excel spreadsheet and lists computer accounts and UUIDs/GUIDs in
 '** an Active Directory domain, in addition to the computer location and
 '** description attributes. .

 Dim Fso
 Dim objExcel
 Dim objSheet
 Dim x
 Dim i

 ON ERROR RESUME NEXT

 '** Create instance of FileSystemObject and get absolute path name for 
 '** Excel file:
     Set Fso = CreateObject("Scripting.FileSystemObject")
     strExcelPath = Fso.GetAbsolutePathName(strExcelPath)

 '** Get instance of Excel application and prepare a spreadsheet:
     Set objExcel = CreateObject("Excel.Application")
  
     '** Check for success of creating object instance:
         If Err.Number <> KNOERROR Then
         WScript.Echo "Error0x" & Hex(Err.Number) & " occurred creating Excel object: you" & _
                      " may not have Excel installed on your system. . ."

         Err.Clear
         WScript.Quit()
         End If
        
     objExcel.Workbooks.Add
     Set objSheet = objExcel.ActiveWorkbook.Worksheets(1)
     objSheet.Cells.Font.Size = 8
     objSheet.Name = "COMPUTER PRESTAGING DATA"
     objSheet.Cells(1,1).Value = "Active Directory Domain:  " & strDomain
     objSheet.Cells(1,1).Font.Bold = True
     objSheet.Cells(1,1).Font.Size = 10
     objSheet.Cells(1,3).Value = "Created by:  " & UCase(strCurrentUser)
     objSheet.Cells(1,3).Font.Bold = True
     objSheet.Cells(1,3).Font.Size = 10
     objSheet.Range("A3:F3").Font.Bold = True
 '** Color the header row gray:
     objSheet.Range("A3:F3").Interior.Color = RGB(192,192,192)
 '** Call procedure to locate cells and write headers:
     SetupColmn objSheet, 3, 1,  25, "Computer Account Name"
     SetupColmn objSheet, 3, 2,  35, "Computer System UUID"
     SetupColmn objSheet, 3, 3,  30, "Location"
     SetupColmn objSheet, 3, 4,  25, "Domain\Username"
     SetupColmn objSheet, 3, 5,  35, "Description"
     SetupColmn objSheet, 3, 6,  60, "Startup File Path"

 '** Initialize a separate counter for spreadsheet cells since the For Loop
 '** control variable (i) has a Step 4 configuration:
     x = 0

     For i = 0 to objDict.Count - 1 Step 4
     objSheet.Cells(4 + x, 1).Value = objDict.Item(i + 0)
     objSheet.Cells(4 + x, 2).Value = objDict.Item(i + 1)
     objSheet.Cells(4 + x, 3).Value = objDict.Item(i + 2)
     objSheet.Cells(4 + x, 5).Value = objDict.Item(i + 3)
     x = x + 1
     Next

 '** Save spreadsheet and close:
     objExcel.ActiveWorkbook.SaveAs strExcelPath
     objExcel.ActiveWorkbook.Close

 '** Release object references:
     Set objSheet = Nothing
     Set objExcel = Nothing
     Set Fso      = Nothing

 End Sub

'*******************************************************************************
 Sub SetupColmn(objSheet, intRow, intCol, intWidth, strTitle)

 '** Sets up the cell columns and headings for the Excel spreadsheet. .

     objSheet.Cells(intRow, intCol).Value = strTitle
     objSheet.Cells(intRow, intCol).ColumnWidth = intWidth

 End Sub

'********************************************************************************
 Sub ShowUsage()

 WScript.Echo "************************ GetRISBIOSInfo ****************************"
 WScript.Echo ""
 WScript.Echo "SCRIPT HELP. . ."
 WScript.Echo ""
 WScript.Echo "PURPOSE:"
 WScript.Echo "1. Retrieves UUIDs of all computers in the Active Directory default" 
 WScript.Echo "   Computers container."
 WScript.Echo "2. Returns BIOS parameters of all computers in Active Directory"
 WScript.Echo "   Computers container to expose support (or no support) for PXE-"
 WScript.Echo "   enabled remote booting."
 WScript.Echo "3. Retrieves the UUID or BIOS information on a specific machine."
 WScript.Echo ""
 WScript.Echo "OPTIONS:"
 WScript.Echo "/Command:Value    Value specifies the action command you want to run:"
 WScript.Echo "                  ""getuuid"" ------ returns a computer system UUID."
 WScript.Echo "                  ""getalluuids"" -- returns UUIDs of domain computers."
 WScript.Echo "                  ""getbios"" ------ returns parameters of a BIOS."
 WScript.Echo "                  ""getallbios"" --- returns BIOS parameters of domain"
 WScript.Echo "                                   computers."
 WScript.Echo "/Server:Value     Value specifies remote server on which to search."
 WScript.Echo "/ExcelPath:Value  Value specifies path to an Excel spreadsheet for"
 WScript.Echo "                  rendering output of UUIDs [optional]. If not used,"
 WScript.Echo "                  output defaults to the command shell."
 WScript.Echo "/LogEvent:Value   Value specifies name of the server where the script"
 WScript.Echo "                  dumps Event Log entries [optional], in case errors"
 WScript.Echo "                  occur in getting all UUIDs or if a UUID is not found."
 WScript.Echo "/?                Displays script usage instructions."
 WScript.Echo ""
 WScript.Echo "USAGE EXAMPLES:"
 WScript.Echo "cscript.exe GetRISBIOSInfo.vbs /Command:getbios [/Server:servername]"
 WScript.Echo "cscript.exe GetRISBIOSInfo.vbs /Command:getuuid [/Server:servername]"
 WScript.Echo "cscript.exe GetRISBIOSInfo.vbs /Command:getallbios [> biosstats.txt]"
 WScript.Echo "cscript.exe GetRISBIOSInfo.vbs /Command:getalluuids [/ExcelPath:spdsht.xls]"
 WScript.Echo "                              [LogEvent:servername]"
 WScript.Echo ""
 WScript.Echo "NOTES:"
 WScript.Echo "1. Do not use double backslashes (\\) when specifying servername."
 WScript.Echo "2. Where servername is not specified, the local computer is assumed."
 WScript.Echo "3. Administrator privileges required in the RIS domain."
 WScript.Echo "4. Requires Excel installed on computer running script."
 WScript.Echo "5. Script requires WSH version 5.6."
 WScript.Echo ""
 WScript.Echo "*********************************************************************"

 End Sub



