'*********************************************************************************
'*
'*
'*
'*					Disclaimer
'*
'** This sample script is not supported under any Microsoft standard support program 
'** or service. This sample script is provided AS IS without warranty of any kind. 
'** Microsoft further disclaims all implied warranties including, without limitation,
'** any implied warranties of merchantability or of fitness for a particular purpose.
'** The entire risk arising out of the use or performance of this sample script and
'** documentation remains with you. In no event shall Microsoft, its authors, or 
'** anyone else involved in the creation, production, or delivery of the script be
'** liable for any damages whatsoever (including, without limitation, damages for 
'** loss of business profits, business interruption, loss of business information,
'** or other pecuniary loss) arising out of the use of or inability to use this 
'** sample script or documentation, even if Microsoft has been advised of the 
'** possibility of such damages. 
'**
'**
'**                          Copyright 2003, Microsoft Corporation
'**
'**
'********************************************************************************

'********************************************************************************
'** DESCRIPTION: Sample script showing how to prestage RIS clients in Active 
'** Directory directory service.
'**
'** REFERENCE: Chapter 4 "Designing RIS Installations" in the Windows Server 2003
'** Deployment Kit: Automating and Customizing Installations
'**
'** SAMPLE Script Name: PrestagRISClnt.txt
'**
'** This sample script shows how to prestage client computers in Active Directory
'** directory service. This sample script relies on a .dll (GUIDConvert.dll) which
'** you must create. For information about how to create this .dll, see article Q829032
'** "Cannot Prestage RIS Clients in Active Directory By Using the Windows Server 2003 
'** Deployment Kit" in the Microsoft Knowledge Base. This script also relies on a 
'** spreadsheet which contains the UUIDs/GUIDs, computer names, and other information that
'** you need to add to Active Directory. You can create this spreadsheet manually 
'** or you can use the sample script "GetRISBIOSInfo.txt" to create this spreadsheet. 
'** If you modify the spreadsheet, be sure that the data entry starts in Row 4 (Rows 1 - 3 are 
'** reserved for headings and other information). The columns in this spreadsheet can
'** contain the following information, which correspond to computer object
'** properties/attributes in Active Directory:
'** 
'** Column 1 - Computer Account Name
'** Column 2 - Computer System UUID/GUID
'** Column 3 - Computer Location
'** Column 4 - Domain\Username
'** Column 5 - Computer Description
'** Column 6 - Startup File Path 
'**
'** This sample script reads the values from the spreadsheet and adds them to Active
'** Directory. You must specify values in Columns 1 and 2 to prestage clients. Columns
'** 3, 4 and 5 are optional and do not require data for the script to run. If you 
'** specify a value for Column 4,the script will configure the specified user account 
'** with "Full Control" permissions on the prestaged computer account and will create
'** a password using the machine account name with "$" appended to it. Column 6 is used
'** to specify the path to the startup file that you want a client to use: either 
'** startrom.com for interactive installations, or startrom.n12 for automated 
'** installations. If you want to require all clients to use the same startup file, 
'** you can leave Column 6 blank and use the /command switch to specify an installation
'** method.
'** 
'** NOTES:
'**
'** There cannot be any blank rows between computer account listings 
'** in the spreadsheet, starting with Row 4.
'** 
'** Before prestaging, delete existing computer accounts from the Active Directory
'** default Computers container (after creating the Excel spreadsheet with the 
'** prestaging data).
'**
'** Do not prestage the RIS server computer account. Remove it from the Excel 
'** spreadsheet if you used the sample script "GetRISBIOSInfo.txt" to populate
'** the spreadsheet.
'**
'** Make sure that GUIDConvert.dll is registered on the system where the script
'** runs, so it can be accessed as a COM object from the script. 
'** 
'** SYNTAX:
'**     
'** Cscript.exe PreStagRISClnt.vbs /InputFile:InputFilePathAndName /Server:ServerName 
'**   [/Command:[automate | interactive]]
'**
'**   /InputFile:InputFilePathAndName
'**      Specifies the path and file name of the Excel file containing the computer
'**      names, UUIDs/GUIDs, and startup file path for the computers you want to prestage
'**      in Active Directory.
'** 
'**   /Server:ServerName
'**      Specifies the name of the RIS server that contains the startup files.
'**      ServerName must not contain \\.
'**
'**   [/Command:[automate | interactive]]
'**     Specifies whether you want all RIS clients to use startrom.com (interactive
'**     installation) or startrom.n12 (automated installation). This is an
'**     optional setting. If you do not use this setting then you must specify
'**     the installation type for each client in the Excel spreadsheet. If you use
'**     this option, you must use the /Server option.
'**
'**
'********************************************************************************

'********************************************************************************
 OPTION EXPLICIT

 Dim strSpace
 Dim strRetValHst
 Dim intRetVal
 Dim lFlag
 Dim Fso
 Dim objADsArrayConvert
 Dim objArgs
 Dim objNamed
 Dim strXLFileName
 Dim strCommand
 Dim strServer
 Dim intConfirm
 Dim objExcel
 Dim strExcelPath
 Dim objSheet
 Dim objRange
 Dim x
 Dim intFormat
 Dim intMsg

 '** Declare constants used in defining the default location for the machine 
 '** account, flags to identify the object as a machine account, and security 
 '** flags:
     CONST ADS_GUID_COMPUTRS_CONTAINER  = "aa312825768811d1aded00c04fd8d5cd"
     CONST UF_WORKSTATION_TRUST_ACCOUNT = &H1000
     CONST UF_ACCOUNTDISABLE            = &H2
     CONST UF_PASSWD_NOTREQD            = &H20
     CONST ADS_ACETYPE_ACCESS_ALLOWED   = 0
     CONST ADS_ACEFLAG_INHERIT_ACE      = 2

 '** Declare "invalid input" constants:
     CONST KINVALID_1     = 1
     CONST KINVALID_2     = 2
     CONST KINVALID_3     = 3
     CONST KINVALID_4     = 4
     CONST KINVALID_5     = 5
     CONST KINVALID_6     = 6
     CONST KINVALID_7     = 7
     CONST KINVALID_8     = 8

 '** Declare "input command" constants:
     CONST KUNSPECIFIED   = "unspecified"
     CONST KAUTOMATE      = "automate"
     CONST KINTERACTIVE   = "interactive"

 '** Declare various other constants:
     CONST KNOERROR       =  0
     CONST KMSG_CANCEL    =  2
     CONST KRETVAL_YES    =  6
     CONST KRETVAL_NO     =  7
     CONST KFORMAT_10     = 10
     CONST KFORMAT_11     = 11
     CONST KUNDETERMINED  = "undetermined"
     CONST KERROR         = "error"
     CONST KSUCCESS       = "success"
     CONST KFAILURE       = "failure"
     CONST KQUOTE         = """"
     CONST KTRUE          = True
     CONST KFALSE         = False

 lFlag = UF_WORKSTATION_TRUST_ACCOUNT Or UF_ACCOUNTDISABLE Or UF_PASSWD_NOTREQD

 ON ERROR RESUME NEXT

 '**
 '** Main execution begins:
 '**

 '** ////////////////////////////////////////////
 '** Create spacer for formatting output message:

     If Len(Time) = 10 Then
     strSpace = "    "
     ElseIf Len(Time) = 11 Then
     strSpace = "     "
     End If

 '** ////////////////////////////////////////////////////
 '** Call the function to detect the default script host:
     strRetValHst = IsHostCScript()

 '** ////////////////////////////////////
 '** Check return value of IsHostCScript:

     Select Case strRetValHst

            Case KERROR
            intRetVal = Msgbox _
            ("The default WSH script host was not detected"            & vbcrlf & _
             "since Error0x" & Hex(Err.Number)                                  & _
             " occurred unexpectedly !"                                & vbcrlf & _
             "Error description: " & Err.Description & vbcrlf          & vbcrlf & _
             "If you continue, the script might fail to execute. . "   & vbcrlf & _
             "Do you want to continue anyway ?", 20, "Script Host Error")

                If intRetVal = KRETVAL_YES Then
            '** Do nothing and proceed with script execution...
                Err.Clear

                ElseIf intRetVal = KRETVAL_NO Then
                Err.Clear
                WScript.Quit()
                End If

            Case KUNDETERMINED
            Msgbox "Could not determine default script host, "                  & _
                   "the script may behave" & vbcrlf & "unexpectedly. . .",        _
                    64, "Script Host Message"

            Case KFAILURE
        '** If default script host is set to WScript, if the script is launched
        '** by double-clicking it, or if user happens to run the script with 
        '** WScript.exe at the command line, then send a message to the user to 
        '** change the default host:

            Msgbox "Script requires CSCRIPT.EXE to execute !"          & vbcrlf & _
                   "Either specify cscript.exe in the command line"    & vbcrlf & _
                   "or change the default script host to cscript.exe"  & vbcrlf & _
                   "by running the following command:"        & vbcrlf & vbcrlf & _
                   "cscript.exe //H:cscript //S", 64, "Script Host Message"

            WScript.Quit()

            Case KSUCCESS
        '** Do nothing to proceed...

            Case Else
            Msgbox "Could not determine default script host, "                  & _
                   "the script may behave" & vbcrlf & "unexpectedly. . .",        _
                    64, "Script Host Message"

     End Select

 '** //////////////////////
 '** Create global objects:

 '** Create FileSystemObject:
     Set Fso = CreateObject("Scripting.FileSystemObject")

 '** Create GUIDConvert object:
     Set objADsArrayConvert = CreateObject("GUIDConvert.Convert")

         If Err.Number <> KNOERROR Then
         WScript.Echo "Error0x" & Hex(Err.Number) & _
                      " occurred creating GUIDConvert object. . ."

             If Err.Description > "" Then
             WScript.Echo Err.Description
             WScript.Echo "Make sure guid.dll is registered on your system !"
             End If

         Err.Clear
         WScript.Quit()
         End If

 '** Create Named arguments object:
     Set objArgs  = WScript.Arguments
     Set objNamed = objArgs.Named

 '** /////////////////////////////////
 '** Check if user is asking for Help:

     If objArgs.Count <> 0 Then

         If objArgs.Item(0) = "?"     OR _
            objArgs.Item(0) = "/?"    OR _
            objArgs.Item(0) = "-h"    OR _
            objArgs.Item(0) = "-H"    OR _
            objArgs.Item(0) = "-help" OR _
            objArgs.Item(0) = "-Help" OR _
            objArgs.Item(0) = "help"  OR _
            objArgs.Item(0) = "Help"  OR _
            objArgs.Item(0) = "/help" OR _
            objArgs.Item(0) = "/Help" Then

         ShowUsage()
         WScript.Quit(1)
         End If

     ElseIf objArgs.Count = 0 Then
     Call Abort(KINVALID_1)

     End If

 '** ///////////////////////////////////////
 '** Check the input argument configuration:

 '** If the user is setting the same startup file for all prestaged RIS 
 '** clients, get the input action command. .
     If objNamed.Exists("Command") Then
     strCommand = LCase(objNamed("Command"))

         If strCommand = KINTERACTIVE or strCommand = KAUTOMATE Then

         '** Get server name for specifying path to startup boot file to 
         '** use on all clients prestaged with this script:
             If objNamed.Exists("Server") Then
             strServer = "\\" & LCase(objNamed("Server"))

                 If strServer = "\\" OR InStr(strServer, "\\\\") <> 0 Then
                 Call Abort(KINVALID_5)
                 End If

         '** Call procedure to check if a connection to the specified server can be made:
             Call CheckServer(KFORMAT_10)

             Else
             Call Abort(KINVALID_3)

             End If

         '** Get path to input file:
             If objNamed.Exists("InputFile") Then
             strXLFileName = LCase(objNamed("InputFile"))

                 If strXLFileName = "" Then
                 Call Abort(KINVALID_2)

                 ElseIf InStr(strXLFileName, ".xls") = 0 Then
                 Call Abort(KINVALID_6)
                 End If

             Else
             Call Abort(KINVALID_2)

             End If

         Else
         Call Abort(KINVALID_4)
        
         End If

 '** /////////////////////////////////////////////////////////////////////
 '** . . or if the user is not specifying a command, get the path to input 
 '** file and get the server name (minimum input argument configuration). .

     ElseIf objArgs.Count = 2            AND _
            objNamed.Exists("InputFile") AND _
            objNamed.Exists("Server")    Then

         strXLFileName = LCase(objNamed("InputFile"))

             If InStr(strXLFileName, ".xls") = 0 Then
             Call Abort(KINVALID_6)
             End If

         strServer = LCase(objNamed("Server"))

             If strServer = "" OR InStr(strServer, "\\") <> 0 Then
             Call Abort(KINVALID_5)
             End If

     '** Call procedure to check if a connection to the specified server can be made:
         Call CheckServer(KFORMAT_11)

         strCommand  = KUNSPECIFIED

     Else
 '** . . otherwise, the input argument configuration is incorrect:
     Call Abort(KINVALID_8)

     End If

 '** ///////////////////////////////////////////////////////////////////
 '** Call procedure to check if specified input file actually exists and 
 '** get count for number of computer accounts being added:
     Call CheckInputFile(x, strXLFileName)

     If x > 0 Then

 '** Prompt user with a continuation message:
     intConfirm = Msgbox _
     ("You are about to add " & x & " machine accounts to Active " & _
      "Directory. Click OK to" & vbcrlf & "continue, or press "    & _
      "Cancel if you do not want to add these accounts now. . .",    _
       vbOkCancel, "Add Machine Accounts ?")

         If intConfirm = KMSG_CANCEL Then
         WScript.Echo Time & ": Aborted prestaging process !"
         WScript.Quit()

         Else
         WScript.Echo Time & ": Processing prestaging data. . ."
         WScript.Echo ""

         Call CreateComputerAccts()

         End If

     Else
     Call Abort(KINVALID_7)
     End If

 '** Release object references:
     Set objNamed           = Nothing
     Set objArgs            = Nothing
     Set objADsArrayConvert = Nothing
     Set Fso                = Nothing

 '**
 '** End main execution
 '**

'********************************************************************************
 Function IsHostCScript()
 
 '** Detects if WSH default host is cscript.exe. .

 Dim strHostPath
 Dim strHostName

 ON ERROR RESUME NEXT

 '** Initialize string variables:
     strHostPath = ""
     strHostName = ""

 '** Get default script host name:
     strHostPath = LCase(WScript.FullName)

 '** Check if any errors occurred (may happen if WSH is not installed):
     If Err.Number <> KNOERROR Then
     IsHostCScript = KERROR
     Exit Function

 '** In case strHostPath variable was not set by WScript.FullName:
     ElseIf strHostPath = "" OR IsEmpty(strHostPath) Then
     IsHostCScript = KUNDETERMINED
     Exit Function

     ElseIf InStr(strHostPath, "cscript") = 0 Then
     IsHostCScript = KFAILURE
     Exit Function

     ElseIf InStr(strHostPath, "cscript") <> 0 Then
     strHostName = Trim(Right(strHostPath, 11))

         If strHostName = "cscript.exe" Then
         IsHostCScript = KSUCCESS

         Else
         IsHostCScript = KFAILURE
         End If

     End If

 End Function

'********************************************************************************
 Sub Abort(intMsg)

 '** Displays appropriate message if user specifies an incorrect input 
 '** argument, command configuration, or other invalid input data . . .
 
     If intMsg     = 1 Then
     WScript.Echo "Aborted, input arguments required, see script Help. ."

     ElseIf intMsg = 2 Then
     WScript.Echo "Input file required to run the script, see script Help. ." 

     ElseIf intMsg = 3 Then            
     WScript.Echo "Aborted, server name required, see script Help. ."

     ElseIf intMsg = 4 Then
     WScript.Echo "Aborted, invalid action command, see script Help. ."

     ElseIf intMsg = 5 Then
     WScript.Echo "Aborted, invalid servername format, see script Help. ."

     ElseIf intMsg = 6 Then
     WScript.Echo "Aborted, please specify the .xls extension "        & _
                  "for your input Excel file !"
     WScript.Quit()

     ElseIf intMsg = 7 Then
     WScript.Echo "The input file " & KQUOTE & strXLFileName & KQUOTE  & _
                  " has no valid data for prestaging computer accounts. . ."
     WScript.Quit()

     ElseIf intMsg = 8 Then
     WScript.Echo "Aborted, invalid input argument configuration, see script Help. ." 
     End If

     WScript.Echo ""
     ShowUsage()
     WScript.Quit()

 End Sub

'********************************************************************************
 Sub CheckInputFile(ByRef x, strXLFileName)

 '** Checks for the presence of the input file and gets the count of the
 '** number of computer accounts in the Excel spreadsheet. . .

 Dim u
 Dim Item

 ON ERROR RESUME NEXT

 '** Check for existence of input file:
     If Not (Fso.FileExists(strXLFileName)) Then
     WScript.Echo "The input file " & KQUOTE & strXLFileName & KQUOTE  & _
                  " is missing. Please locate the file and place it "  & _
                   vbCrLf                                              & _
                  "in the appropriate directory location before "      & _
                  "running this script. ."
     WScript.Quit()
     End If

 '** Create object instance of the Excel application for verifying the 
 '** input data:
     Set objExcel = CreateObject("Excel.Application")
  
     '** Check for success of creating object instance:
         If Err.Number <> KNOERROR Then
         WScript.Echo "Error0x" & Hex(Err.Number) & " occurred creating Excel " & _
                      "object. . " & vbcrlf                                     & _
                      "You may not have Excel installed on your system !"

         Err.Clear
         WScript.Quit()
         End If

     strExcelPath = Fso.GetAbsolutePathName(strXLFileName)
     objExcel.Workbooks.Open(strExcelPath)
     Set objSheet = objExcel.ActiveWorkbook.Worksheets(1)

 '** Create cell collection and count cell values in first column to get
 '** number of computer accounts being added from the Excel spreadsheet
 '** and verify that the UUID/GUID for each computer is valid:

     Set objRange = objSheet.Cells

     u = 4

     For Each Item in objRange

     '** Exit loop when a blank cell in the first column is encountered
     '** after the listings start in the 4th Row.
     '** Exit program if an invalid UUID is found:

         
         If objRange.Cells(u, 1) = "" Then
           x = u - 4
           Exit For
         ElseIf Len(objRange.Cells(u,2)) <> 36 Then
           WScript.Echo "The input file " & KQUOTE & strXLFileName & KQUOTE & _
                  " contains" & vbCrLf & "an invalid computer system UUID."  & _
                  vbCrLf & "UUIDs must be in the following hexadecimal form: " & _
                  vbCrLf & "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX."
           WScript.Quit()              
         End If

         u = u + 1
     Next

     Set objRange = Nothing

 End Sub

'********************************************************************************
 Sub CheckServer(intFormat)

 '** Checks if a connection can be made to the specified RIS server and verifies
 '** whether a remote installation share (REMINST) exists there. .

 Dim intSlshPos
 Dim strRISServer
 Dim objService
 Dim objShare
 Dim Share
 Dim blnShareExists

 ON ERROR RESUME NEXT

 blnShareExists = KFALSE

 '** Get the server name in the correct format for GetObject:
     If intFormat = KFORMAT_10 Then
     intSlshPos = InStr(strServer, "\\")

         If intSlshPos <> 0 Then
         strRISServer = Mid(strServer, intSlshPos + 2, Len(strServer) - (intSlshPos + 1))
         End If

     ElseIf intFormat = KFORMAT_11 Then
     strRISServer = strServer
     End If

 '** Connect with CIM repository on RIS server and get instance of 
 '** Winmgmts service:
     Set objService = GetObject("Winmgmts:\\" & strRISServer & "\root\cimv2")

     If Err.Number <> KNOERROR Then
     WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred attempting to " & _
                         "connect with Winmgmts on " & UCase(strServer) & ". . ."

         If Err.Description > "" Then
         WScript.Echo vbTab & strSpace & Err.Description & " !"
         End If

     Err.Clear
     WScript.Quit()

     Else
 '** Get instance of Win32_Share class:
     Set objShare = objService.ExecQuery("Select * from Win32_Share")

         For Each Share in objShare

             If Share.Name = "REMINST" Then
             blnShareExists = KTRUE
             Exit For
             End If

         Next
     
         If blnShareExists = KFALSE Then
         WScript.Echo Time & ": The server you specified does not "     & _
                     "have a " & KQUOTE & "REMINST" & KQUOTE            & _
                     " share ! " & vbcrlf                               & _
                      vbTab & strSpace & "Please check the server name. . ."
         WScript.Quit()
         End If

     End If

 End Sub

'********************************************************************************
 Sub CreateComputerAccts()

 '** Creates prestaged computer accounts in Active Directory along with  
 '** configuration of the netbootGUID attribute, the RIS server that services 
 '** the client, and the netbootSIFFile attribute (startupfile). . .

 Dim bolErrFlag
 Dim strCompFail
 Dim u
 Dim Item
 Dim strMAOName
 Dim strUUID
 Dim strLocation
 Dim strUser
 Dim strDescription
 Dim strStartup
 Dim rootDSE
 Dim compContainer
 Dim arrayByteOctel
 Dim objComputer
 Dim secDescriptor
 Dim dACL
 Dim ACE

 ON ERROR RESUME NEXT

 '** Create cell collection:
     Set objRange = objSheet.Cells

     bolErrFlag = False
     strCompFail = ""
     u = 4

     For Each Item in objRange

         If objRange.Cells(u, 1) = "" Then
         Exit For

         Else
     
         strMAOName     =         objRange.Cells(u, 1)
         strUUID        = Replace(objRange.Cells(u, 2), "-","")
         strLocation    =         objRange.Cells(u, 3)
         strUser        =         objRange.Cells(u, 4)
         strDescription =         objRange.Cells(u, 5)
         
             If strCommand = KINTERACTIVE Then
             strStartup = strServer & "\REMINST\OSChooser\i386\startrom.com"
 
             ElseIf strCommand = KAUTOMATE Then
             strStartup = strServer & "\REMINST\OSChooser\i386\startrom.n12" 

             ElseIf strCommand = KUNSPECIFIED Then
             strStartup = objRange.Cells(u, 6)
             End If

          End If

 '** ///////////////////////////////////////////////////////////////////
 '** Establish a path to the container in the Active Directory where the
 '** machine account will be created. The following code automatically 
 '** locates a domain controller for the domain, reads the distinguished 
 '** name, and binds to the default "Computers" container:

     Set rootDSE = GetObject("LDAP://RootDSE")
     Set compContainer = GetObject("LDAP://<WKGUID=" & ADS_GUID_COMPUTRS_CONTAINER & _
                                   "," & rootDSE.Get("defaultNamingContext") & ">")
     Set compContainer = GetObject("LDAP://" & compContainer.Get("distinguishedName"))

         If Err.Number <> KNOERROR AND Hex(Err.Number) <> 80071392 Then
         WScript.Echo Time & ": Error0x" & Hex(Err.Number)            & _
                             " occurred getting AD default " & KQUOTE & _
                             "Computers" & KQUOTE & " object !"
  
             If Err.Description > "" Then
             WScript.Echo vbTab & strSpace & Err.Description
             End If

         Err.Clear
         WScript.Quit()
         End If

 '** //////////////////////////////////////////////////////////////////////
 '** Create the computer account and set attributes with appropriate values 
 '** before calling SetInfo to commit changes to Active Directory:

 '** Convert UUID/GUID to a byte array:
     arrayByteOctel = objADsArrayConvert.MakeGUIDByteArray(strUUID)

 '** Create computer account attributes:
     Set objComputer = compContainer.Create("computer", "CN=" & strMAOName)
     objComputer.Put "samAccountName", strMAOName + "$"
     objComputer.Put "userAccountControl", lFlag
     objComputer.Put "netbootMachineFilePath", strServer
     objComputer.Put "netbootGUID", arrayByteOctel
     objComputer.Put "description", strDescription
     objComputer.Put "location", strLocation
     objComputer.Put "netbootSIFFile", strStartup
     objComputer.AccountDisabled = FALSE

     objComputer.SetInfo

     '** Check if any errors occur when calling the SetInfo method:
         If Hex(Err.Number) = 80071392 Then
         WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred while " & _
                      "processing " & KQUOTE & strMAOName & KQUOTE              & _
                      vbcrlf                                                    & _
                      vbTab & strSpace & "The computer object already exists !"
         WScript.Echo ""
         bolErrFlag  = True
         strCompFail = strCompFail & vbTab & strSpace & strMAOName & vbcrlf
         Err.Clear

         ElseIf Err.Number <> KNOERROR Then
         WScript.Echo Time & ": Error0x" & Hex(Err.Number) & _
                      " occurred while processing "        & _
                      KQUOTE & strMAOName & KQUOTE & " !"

             If Err.Description > "" Then
             WScript.Echo vbTab & strSpace & Err.Description
             End If
         
         WScript.Echo ""
         bolErrFlag  = True
         strCompFail = strCompFail & vbTab & strSpace & strMAOName & vbcrlf 
         Err.Clear
         End If

 '** //////////////////////////////////////////////////
 '** Create a default password for the machine account:
     objComputer.SetPassword LCase(strMAOName & "$")

 '** //////////////////////////////////////////////////////////////////
 '** Bind to the Discretionary ACL on the newly created computer account
 '** and create an Access Control Entry (ACE) that gives the specified
 '** user or group full control on the machine account:

     Set secDescriptor = objComputer.Get("ntSecurityDescriptor")
     Set dACL          = secDescriptor.DiscretionaryAcl
     Set ACE           = CreateObject("AccessControlEntry")

 '** Set an AccessMask of "-1" to grant Full Control:
     ACE.AccessMask = -1
     ACE.AceType    = ADS_ACETYPE_ACCESS_ALLOWED
     ACE.AceFlags   = ADS_ACEFLAG_INHERIT_ACE

 '** Grant this control to the appropriate user or group. This uses 
 '** the downlevel naming convention (as in DOMAIN\User):
     ACE.Trustee = strUser

 '** Add the ACE to the DACL on the machine account:
     dACL.AddAce ACE
     secDescriptor.DiscretionaryAcl = dACL

 '** Commit the security changes to the machine account:
 '** The following line is generating an error:
     objComputer.Put "ntSecurityDescriptor", Array(secDescriptor)
     objComputer.SetInfo

 '** Enable the account:
     objComputer.AccountDisabled = False
     objComputer.SetInfo

     u = u + 1

     Err.Clear

     Next

 '** Close spreadsheet:
     objExcel.ActiveWorkbook.Close

     Set objRange = Nothing
     Set objSheet = Nothing
     Set objExcel = Nothing

 '** //////////////////////////////////////////////////////////////
 '** Echo success if no errors occurred (bolErrFlag will be False):

     If bolErrFlag = False Then
     WScript.Echo Time & ": Success creating new computer accounts in " & _
                         "Active Directory based on UUIDs !"
    
     ElseIf bolErrFlag = True Then
     WScript.Echo Time & ": The following computer accounts could not be" & _
                        " created in Active Directory:" & vbcrlf & vbcrlf & strCompFail
     End If

 End Sub

'********************************************************************************
 Sub ShowUsage()

 WScript.Echo "*********************** PreStagRISClnt.txt **********************"
 WScript.Echo ""
 WScript.Echo "SCRIPT HELP. . ."
 WScript.Echo ""
 WScript.Echo "PURPOSE:"
 WScript.Echo "1. Prestage computer accounts in the default Computers container in"
 WScript.Echo "   Active Directory." 
 WScript.Echo "2. Explicitly specify a startup file for all prestaged RIS clients."
 WScript.Echo ""
 WScript.Echo "ARGUMENTS:"
 WScript.Echo "/InputFile:Value    Value specifies the path to an Excel spreadsheet"
 WScript.Echo "                    file containing the prestaging and startup file"
 WScript.Echo "                    data."
 WScript.Echo "/Server:Value       Value specifies name of remote RIS server that"
 WScript.Echo "                    contains the startup boot files."
 WScript.Echo "[/Command:Value]    Value specifies the action command to run:"
 WScript.Echo "                    ""interactive"" -- sets all prestaged RIS clients"
 WScript.Echo "                                     to use default startrom.com."
 WScript.Echo "                    ""automate"" ----- sets all prestaged RIS clients"
 WScript.Echo "                                     to use startrom.n12."
 WScript.Echo "/?                  Displays script usage instructions."
 WScript.Echo ""
 WScript.Echo "USAGE EXAMPLE:"
 WScript.Echo "cscript.exe prestagRisclnt.vbs /InputFile:filepath /Server:servername"
 WScript.Echo "                               [/Command:automate | interactive]"
 WScript.Echo ""
 WScript.Echo "NOTES:"
 WScript.Echo "1. Do not use double backslashes (\\) when specifying servername."
 WScript.Echo "2. Use quotes for any named argument values that contain spaces."
 WScript.Echo "3. You must generate the input file (spreadsheet) manually or by "
 WScript.Echo "   using the sample script GetRISBIOSInfo.txt."
 WScript.Echo "4. If Command is specified, it overrides the ""startup file path"""
 WScript.Echo "   in the input spreadsheet and configures all clients with the same"
 WScript.Echo "   startup file."
 WScript.Echo "5. Script requires WSH version 5.6."
 WScript.Echo "6. Admin privileges required in the RIS domain."
 WScript.Echo ""
 WScript.Echo "*********************************************************************"

 End Sub


